#include "game_display.h"
#include "actions.h"
#include "player.h"
#include "options.h"
#include <unistd.h>

#ifndef DEBUG
WINDOW *game_win=NULL;

void game_win_create()
{
    game_win=newwin(GAREA_H*ICON_H, GAREA_W*ICON_W, 0, 0);
    scrollok(game_win,FALSE);
    nodelay(game_win,FALSE);
    keypad(game_win,TRUE);
    init_pair(ICON_XXX+1,COLOR_BLACK,COLOR_YELLOW);
}

void game_win_destroy()
{
    delwin(game_win);
}
#endif

void game_win_display()
{
    xy_t ox,oy;
    xy_t x,y;
    ox=map->view[player.player].x;
    oy=map->view[player.player].y;
    for(y=0;y<GAREA_H;y++){
        for(x=0;x<GAREA_W;x++){
            #ifndef DEBUG
            game_win_icon_dump(x*ICON_W, y*ICON_H, map->buf[oy+y][ox+x]);
            #else
            putchar(icon_to_mapchar(map->buf[oy+y][ox+x]));
            #endif
        }
        #ifdef DEBUG
        putchar('\n');
        #endif
    }
    #ifndef DEBUG
    wrefresh(game_win);
    #endif
}

void game_win_show(xy_t tlx, xy_t tly)
{
    xy_t x,y;
    for(y=0;y<GAREA_H;y++){
        for(x=0;x<GAREA_W;x++){
            #ifndef DEBUG
            game_win_icon_dump(x*ICON_W, y*ICON_H, map->buf[tly+y][tlx+x]);
            #else
            putchar(icon_to_mapchar(map->buf[tly+y][tlx+x]));
            #endif
        }
        #ifdef DEBUG
        putchar('\n');
        #endif
    }
    #ifndef DEBUG
    wrefresh(game_win);
    #endif
}

void game_win_swap_update()
{
    su_t p=(player.player)?0:1;
    if(map->player[player.player].x>map->view[p].x
     &&map->player[player.player].x<map->view[p].x+GAREA_W-1)
    {
        if(map->player[player.player].y>map->view[p].y
         &&map->player[player.player].y<map->view[p].y+GAREA_W-1){
            map->view[player.player].x=map->view[p].x;
            map->view[player.player].y=map->view[p].y;
        }
    }
}

void game_win_move_player(struct xor_move* pmv)
{
    xy_t vx, vy;
    xy_t ovx,ovy;
    su_t scrt=(player.replay?1:options->scroll_thresh);
    ovx=vx=map->view[player.player].x;
    ovy=vy=map->view[player.player].y;
    switch(pmv->dir){
        case MV_LEFT:
            if(pmv->to_x<vx+scrt&&vx>0)
                vx--;
            break;
        case MV_RIGHT:
            if(pmv->to_x>vx+GAREA_W-1-scrt&&vx<MAP_W-GAREA_W)
                vx++;
            break;
        case MV_UP:
            if(pmv->to_y<vy+scrt&&vy>0)
                vy--;
            break;
        case MV_DOWN:
            if(pmv->to_y>vy+GAREA_H-1-scrt&&vy<MAP_H-GAREA_H)
                vy++;
            break;
        default:break;
    }
    if(ovx!=vx||ovy!=vy){
        map->view[player.player].x=vx;
        map->view[player.player].y=vy;
        game_win_display(map);
        return;
    }
    #ifndef DEBUG
    game_win_icon_display(pmv->from_x, pmv->from_y, ICON_SPACE);
    game_win_icon_display(pmv->to_x, pmv->to_y, pmv->from_obj);
    wrefresh(game_win);
    #else
    game_win_display();
    #endif
}

void game_win_move_object(struct xor_move* omv)
{
    #ifndef DEBUG
    game_win_icon_display(omv->from_x, omv->from_y, ICON_SPACE);
    game_win_icon_display(omv->to_x, omv->to_y, omv->from_obj);
    wrefresh(game_win);
/*
    #else
    game_win_display();
*/
    #endif
}

struct xy* game_win_map_coord(xy_t x, xy_t y)
{
    struct xy* ret=0;
    struct xy* pv=&map->view[player.player];
    if(!(ret=malloc(sizeof(struct xy))))
        return 0;
    if(x >= pv->x && x < pv->x+GAREA_W
     &&y >= pv->y && y < pv->y+GAREA_H)
    {
        ret->x=(x-pv->x)*ICON_W;
        ret->y=(y-pv->y)*ICON_H;
        return ret;
    }
    free(ret);
    return 0;
}

#ifndef DEBUG
void game_win_icon_display(xy_t x, xy_t y, su_t icon)
{
    xy_t xx, yy;
    xy_t wx, wy;
    xy_t vx,vy;
    vx=map->view[player.player].x;
    vy=map->view[player.player].y;
    if(x<vx||x>=vx+GAREA_W||y<vy||y>=vy+GAREA_H)
        return;
    wx=(x-vx)*ICON_W;
    wy=(y-vy)*ICON_H;
    wattrset(game_win,COLOR_PAIR(icon));
    for(yy=0;yy<ICON_H;yy++)
        for(xx=0;xx<ICON_W;xx++)
            mvwaddch(game_win,wy+yy,wx+xx,icons[icon].chrs[yy][xx]);
}

void game_win_icon_dump(xy_t x, xy_t y, su_t icon)
{
    xy_t xx, yy;
    wattrset(game_win,COLOR_PAIR(icon));
    for(yy=0;yy<ICON_H;yy++)
        for(xx=0;xx<ICON_W;xx++)
            mvwaddch(game_win,y+yy,x+xx,icons[icon].chrs[yy][xx]);
}


void game_win_dump_map_sect(xy_t topy, su_t sect, bool show)
{
    xy_t mapstx,mapsty,mapmaxx,mapmaxy;
    xy_t x,y,icon;
    xy_t sy;
    switch(sect){
        case 0:
            mapstx=HMAP_W;
            mapsty=topy;
            mapmaxx=MAP_W;
            mapmaxy=HMAP_H;
            sy=0;
            break;
        case 1:
            mapstx=HMAP_W;
            mapsty=HMAP_H;
            mapmaxx=MAP_W;
            mapmaxy=MAP_H;
            sy=HMAP_H-topy;
            break;
        case 2:
            mapstx=0;
            mapsty=HMAP_H;
            mapmaxx=HMAP_W;
            mapmaxy=MAP_H;
            sy=HMAP_H-topy;
            break;
        case 3:
            mapstx=0;
            mapsty=topy;
            mapmaxx=HMAP_W;
            mapmaxy=HMAP_H;
            sy=0;
            break;
    }
    for(y=mapsty;y<mapmaxy;y++,sy++){
        wattrset(game_win,COLOR_PAIR(ICON_SPACE));
        mvwaddstr(game_win,sy,0,"    ");
        mvwaddstr(game_win,sy,MAP_W+4,"    ");
        for(x=mapstx;x<mapmaxx;x++){
            if(show){
                switch((icon=map->buf[y][x])){
                    case ICON_WALL:
                        wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
                        mvwaddch(game_win,sy,4+x,icon_to_mapchar(icon));
                        break;
                    case ICON_MASK:
                    case ICON_EXIT:
                        wattrset(game_win,COLOR_PAIR(icon));
                        mvwaddch(game_win,sy,4+x,icon_to_mapchar(icon));
                        break;
                    default:
                        wattrset(game_win,COLOR_PAIR(ICON_SPACE));
                        mvwaddch(game_win,sy,4+x,icon_to_mapchar(ICON_SPACE));
                        break;
                }
            }
            else {
                wattrset(game_win,COLOR_PAIR(0));
                mvwaddch(game_win,sy,4+x,' ');
            }
        }
    }
}

void game_win_dump_map(xy_t topy)
{
    su_t i;
    su_t mb=1;
    for(i=0;i<4;i++,mb*=2)
        game_win_dump_map_sect(topy,i,mb&player.have_map);
    wrefresh(game_win);
    return;
}

void game_win_map_display()
{
    int key;
    xy_t y=player.map_view_y;
    xy_t maxscrolly=MAP_W-ICON_H*GAREA_H;
    game_win_dump_map(y);
    while(1){
        key=wgetch(game_win);
        if(key!=ERR){
            switch(key){
                case '\'':
                case KEY_UP:
                    if(y>0) {
                        y--;
                        game_win_dump_map(y);
                    }
                    break;
                case '/':
                case KEY_DOWN:
                    if(y<maxscrolly){
                        y++;
                        game_win_dump_map(y);
                    }
                    break;
                case 'm':
                case 'M':
                    game_win_display(y);
                    player.map_view_y=y;
                    return;
            }
        }
    }
}

#endif
