#include "level_menu.h"

#include "map.h"
#include "game_display.h"
#include "info.h"
#include "scores.h"

#ifndef DEBUG
#include <curses.h>
#endif

#include <string.h>
#include <stdlib.h>

#define LVL_NAME_CHARS 24

lvl_t level_menu(lvl_t level)
{
    su_t min_level=1;   /* do we want to show test level or not? */
    su_t max_level=15;   /* above these are unimplimented */
    su_t i;
    #ifndef DEBUG
    int key;
    #endif
    #ifdef TESTMAP
    min_level=0;
    #endif
    #ifndef DEBUG
    wattrset(game_win,COLOR_PAIR(0));
    mvwprintw(game_win,23,1,"q - quit   l - load replay    h - help");
    wrefresh(game_win);
    wclear(info_win);
    wattrset(info_win,COLOR_PAIR(0));
    box(info_win,0,0);
    mvwprintw(info_win,1,2,"Select level:");
    do{
        for(i=min_level;i<MAX_LEVEL;i++){
            if(i==level)
                wattrset(info_win,COLOR_PAIR(ICON_XXX+1));
            else if (i<=max_level)
                wattrset(info_win,COLOR_PAIR(0));
            else
                wattrset(info_win,COLOR_PAIR(ICON_MASK));
            mvwprintw(info_win,2+i,1," %2d. %s ",i,map_name[i]);
            wattrset(info_win,COLOR_PAIR(0));
            mvwprintw(info_win,2+i,MAPNAME_MAXCHARS+2,"%4d",scores[i]);
        }
        wrefresh(info_win);
        if((key=wgetch(info_win))!=ERR){
            switch(key){
                case '\'':
                case KEY_UP:
                    if(level>min_level)
                        level--;
                    break;
                case '/':
                case KEY_DOWN:
                    if(level<max_level)
                        level++;
                    break;
                case 'q':
                case 'Q':
                    return -1;
                case '\r':
                    return level;
                case 'h':
                case 'H':
                    return 999;
                case 'l':
                case 'L':
                    return 100;
                default:
                    if(key>='0'&&key<='9'){
                        i=9-('9'-key);
                        if(i>=min_level&&i<=max_level)
                            level=i;
                    }
                    break;
            }
        }
    }while(1);
    #else
    char buf[80];
    for(i=min_level;i<MAX_LEVEL;i++)
        printf("level %d: %s\n",i,map_name[i]);
    do{
        printf("level %d - %d? (q to quit)",min_level,max_level);
        scanf("%s",buf);
        if(buf[0]=='q'||buf[0]=='Q')
            return -1;
        level=atoi(buf);
        if(level>=min_level&&level<=max_level)
            return level;
    }while(1);
    #endif
}
#ifndef DEBUG
void help()
{
    char* hf;
    FILE* fp;
    char help[120][43];
    xy_t r,maxrows;
    xy_t row=0;
    int key;
    if(!(hf=options_file_path("xorcurseshelp.txt",options->data_dir)))
        return;
    if((fp=fopen(hf, "r"))){
        while(fgets(help[row++], 42, fp)&&row<120);
        free(hf);
        fclose(fp);
        maxrows=row-1;
    }
    else{
        free(hf);
        icon_help();
        return;
    }
    row=0;
    wclear(game_win);
    wattrset(game_win,COLOR_PAIR(0));
    do{
        for(r=0;r<24;r++)
            mvwprintw(game_win,r,0,help[r+row]);
        switch((key=wgetch(game_win)))
        {
            case KEY_UP:    if(row>0)row--;break;
            case KEY_DOWN:  if(row+24<maxrows)row++;break;
            case 'q':
            case 'Q':
                return;
        }
    }while(key!='h'&&key!='H');
    icon_help();
}

void icon_help()
{
    char* hf;
    FILE* fp;
    char help[50][43];
    xy_t row=0;
    xy_t irow=0;
    xy_t r,maxrows;
    int key;
    su_t i;
    if(!(hf=options_file_path("xorcursesiconhelp.txt",options->data_dir)))
        return;
    if((fp=fopen(hf, "r"))){
        while(fgets(help[row++], 42, fp)&&row<50);
        free(hf);
        fclose(fp);
        maxrows=row-1;
    }
    else{
        free(hf);
        return;
    }
    wclear(game_win);
    row=0;
    do{
        wattrset(game_win,COLOR_PAIR(0));
        for(r=0;r<24;r++)
            mvwprintw(game_win,r,0,help[r+row]);
        for(i=0;i<12;i++){
            game_win_icon_dump(0,i*4-row,i+ICON_H_FIELD);
        }
        irow=(row+1)/4;
        switch((key=wgetch(game_win)))
        {
            case KEY_UP:    if(row>0)row--;break;
            case KEY_DOWN:  if(row+24<maxrows)row++;break;
            case 'q': case 'Q': case 'h': case 'H':
                return;
        }
    }while(key!='q'&&key!='Q');
}

#endif
